package com.emsoft.common

import java.util.regex.Pattern

/**
 * Comparator for strings containing digits. Natural sorting. Split strings on parts and compares each parts
 * separately.
 * The Alphanum Algorithm is discussed at {@linkplain 'http://www.DaveKoelle.com'}
 *
 * @author Vladimir Zhukov
 */
class AlphanumComparator implements Comparator<String> {

    private boolean ignoreCase
    private Pattern pattern

    AlphanumComparator(boolean ignoreCase = true) {
        this.ignoreCase = ignoreCase
        pattern = Pattern.compile("[0-9]+")
    }

    @Override
    int compare(String s1, String s2) {
        def chunks1 = StringHelper.split(s1, pattern)
        def chunks2 = StringHelper.split(s2, pattern)
        int minLength = Math.min(chunks1.size(), chunks2.size())
        for (int i in 0..<minLength) {
            String chunk1 = chunks1[i]
            String chunk2 = chunks2[i]
            int result
            // if parts is digits, then compare their as digits
            if (Character.isDigit(chunk1.charAt(0)) && Character.isDigit(chunk2.charAt(0))) {
                result = Integer.valueOf(chunk1).compareTo(Integer.valueOf(chunk2))
            } else {
                // else compare their as strings
                result = (ignoreCase ? chunk1.compareToIgnoreCase(chunk2) : chunk1.compareTo(chunk2))
            }
            if (result != 0)
                return result
        }
        return s1.length() - s2.length()
    }
}