﻿using System.Collections.Specialized;
using System.Configuration;
using System.Linq;
using Alseco.Utils;

namespace Alseco.Configuration {

    public class AppConfiguration : ConfigurationBase {

        public const string AppConfigurationSectionName = "appConfiguration";

        public AppConfiguration(
            System.Configuration.Configuration configuration,
            ConfigurationUserLevel level, string sectionGroupName,
            string defaultSectionName)
            : base(configuration, level, sectionGroupName, defaultSectionName) {
        }

        public AppConfiguration(string configFileFullName,
            ConfigurationUserLevel level, string sectionGroupName,
            string defaultSectionName)
            : base(configFileFullName, level, sectionGroupName, defaultSectionName) {
        }

        public AppConfiguration(ConfigurationUserLevel level)
            : base(level, AppConfigurationSectionName) {
        }

        public AppConfiguration(bool userLevel)
            : this(userLevel ? ConfigurationUserLevel.PerUserRoamingAndLocal :
            ConfigurationUserLevel.None) {
        }

        protected virtual void CheckParameters(string sectionName,
            NameValueCollection values) {
            ExceptionHelper.CheckStringOnNullOrEmpty(sectionName, "sectionName");
            ExceptionHelper.CheckOnNull(values, "values");
            /*if (CollectionHelper.IsNullOrEmpty(values))
                ExceptionHelper.ThrowEmptyCollection("values");*/
        }

        public void SaveConfiguration(string sectionName, NameValueCollection values) {
            CheckParameters(sectionName, values);
            ConfigurationSectionGroup group =
                (Level == ConfigurationUserLevel.None
                ? (ConfigurationSectionGroup)OpenSectionGroup<ApplicationSettingsGroup>(Configuration, SectionGroupName)
                : (ConfigurationSectionGroup)OpenSectionGroup<UserSettingsGroup>(Configuration, SectionGroupName));
            AppSettingsSection section = OpenSection<AppSettingsSection>(
                group, Level, sectionName);
            foreach (string key in values.Keys) {
                if (section.Settings[key] == null)
                    section.Settings.Add(key, values[key]);
                else
                    section.Settings[key].Value = values[key];
            }
            Configuration.Save();
        }

        public void SaveConfiguration(NameValueCollection values) {
            SaveConfiguration(DefaultSectionName, values);
        }

        public void LoadConfiguration(string sectionName, NameValueCollection values) {
            CheckParameters(sectionName, values);
            AppSettingsSection section = OpenSection<ConfigurationSectionGroup, AppSettingsSection>(
                Configuration, Level, SectionGroupName, sectionName, false);
            if (section == null)
                return;
            if (values.Count == 0) {
                foreach (string key in section.Settings.AllKeys) {
                    values.Add(key, section.Settings[key].Value);
                }
            }
            else {
                foreach (string key in values.Keys.OfType<string>().ToArray()) {
                    if (section.Settings[key] != null)
                        values[key] = section.Settings[key].Value;
                }
            }
        }

        public void LoadConfiguration(NameValueCollection values) {
            LoadConfiguration(DefaultSectionName, values);
        }
    }
}