package com.emsoft.io

import com.emsoft.common.Assert
import com.emsoft.common.StringHelper

/**
 * Utils for io.
 *
 * @author Vladimir Zhukov
 */
class IOHelper {

    protected IOHelper() {
    }

    static boolean deleteIfExist(String path) {
        def fl = new File(path)
        return (fl.exists() && fl.delete())
    }

    static boolean forceCreateDirectory(String path) {
        Assert.hasText(path, "path")
        def fl = new File(getPath(path))
        if (!fl.exists())
            return fl.mkdirs()
        return false
    }

    static String excludeExtension(String fileName) {
        Assert.hasText(fileName, "fileName")
        def whereDot = fileName.lastIndexOf('.')
        if (whereDot > 0 && whereDot < fileName.length() - 1)
            return fileName.substring(0, whereDot)
        return fileName
    }

    static String getPath(String path) {
        Assert.hasText(path, "path")
        def whereSeparator = path.lastIndexOf(File.separator)
        def whereDot = path.lastIndexOf(".")
        if (whereSeparator > 0 && whereSeparator < whereDot)
            return path.substring(0, whereSeparator)
        return path
    }

    static String getSimpleName(String path) {
        Assert.hasText(path, "path")
        def whereSeparator = path.lastIndexOf(File.separator)
        if (whereSeparator > 0)
            return path.substring(whereSeparator + 1, path.length())
        return path
    }

    /**
     * @param filePath
     * @param rewrite
     * @param fileData Closure that returns byte array or string, or byte array or string
     * @return
     */
    static File getFile(String filePath, boolean rewrite, fileData) {
        Assert.hasText(filePath, "filePath")
        Assert.notNull(fileData, "fileData")
        def file = new File(filePath)
        return getFile(file, rewrite, fileData)
    }

    static File getFile(File file, boolean rewrite, fileData) {
        Assert.notNull(file, "file")
        Assert.notNull(fileData, "fileData")
        if (rewrite || !file.exists()) {
            IOHelper.forceCreateDirectory(file.path)
            def data = (fileData instanceof Closure ? fileData() : fileData)
            if (data instanceof byte[]) {
                file.withDataOutputStream { it.write((byte[]) data) }
            }
            else {
                file.write(data as String)
            }
        }
        return file
    }

    static String joinPath(boolean firstSeparator = false, boolean lastSeparator = false, String... path) {
        if (!path)
            return StringHelper.EMPTY
        def sb = new StringBuilder()
        path.eachWithIndex { it, index ->
            if (StringHelper.hasText(it)) {
                def hasFirst = (it[0] == File.separator)
                def hasPrevLast = (sb.length() > 0 && sb.charAt(sb.length() - 1) == File.separator)
                if ((index > 0 && !hasFirst && !hasPrevLast) || (index == 0 && !hasFirst && firstSeparator)) {
                    sb.append(File.separator)
                }
                else if (hasFirst && hasPrevLast) {
                    sb.deleteCharAt(sb.length() - 1)
                }
                sb.append(it)
                if (lastSeparator && index == path.length - 1 &&
                        sb.charAt(sb.length() - 1) != File.separator) {
                    sb.append(File.separator)
                }
            }
        }
        return sb.toString()
    }

    static boolean isEqualsOrSubdir(String parent, String path, String deltaPath) {
        // deltaPath = String.Empty
        // String delta = String.Empty
        // DirectoryInfo parentDir = new DirectoryInfo(parent)
        // parent = Directory.GetDirectoryRoot(parent)
        // DirectoryInfo di = new DirectoryInfo(path)
        // while (di != null) {
        // if (string.Equals(parentDir.Root.FullName, di.Root.FullName,
        // StringComparison.CurrentCultureIgnoreCase)) {
        // deltaPath = delta
        // return true
        // }
        // delta = (string.IsNullOrEmpty(delta) ? di.Name : Path.Combine(di.Name, delta))
        // di = di.Parent
        // }
        return false
    }
}