package com.emsoft.observer;


import com.emsoft.collections.CollectionHelper;
import com.emsoft.common.Assert;
import com.emsoft.events.EventArgs;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;


/**
 * @param <TObserver>
 * @param <TEventAgr>
 * @author Vladimir Zhukov
 */
public class ObservableImpl<TObserver extends Observer<TEventAgr>, TEventAgr extends EventArgs<?, ?>>
        implements Observable<TObserver, TEventAgr> {

    private List<TObserver> observers;
    private List<TObserver> roObservers;

    public ObservableImpl() {
        observers = new ArrayList<TObserver>();
        roObservers = Collections.unmodifiableList(observers);
    }

    protected List<TObserver> getObservers() {
        return roObservers;
    }

    protected void notifyObservers(Iterable<TObserver> observers, TEventAgr arg) {
        Assert.notNull(observers, "observers");
        for (TObserver item : observers)
            item.notification(this, arg);
    }

    public void notifyObservers(TEventAgr arg) {
        synchronized (this) {
            notifyObservers(observers, arg);
        }
    }

    public void notifyObservers(TEventAgr arg, TObserver... excludedObservers) {
        synchronized (this) {
            notifyObservers(CollectionHelper.exclude(observers, excludedObservers), arg);
        }
    }

    public synchronized void subscribe(TObserver observer) {
        Assert.notNull(observer, "observer");
        if (!observers.contains(observer))
            observers.add(observer);
    }

    public synchronized void unsubscribe(TObserver observer) {
        observers.remove(observer);
    }

    public synchronized void unsubscribeAll() {
        observers.clear();
    }
}