package com.emsoft.templates

import static com.emsoft.exceptions.ExHelper.logAndThrow

/**
 * Class for wrapping methods with checked exceptions. The exception is caught, wrapped in unchecked exception
 * and throw.
 *
 * @author Vladimir Zhukov
 * @param < T >
 */
abstract class InputOperationTemplate<T> {

    /**
     * Execute method operation. If exception occurs, then it wrap in BaseRuntimeException and throw.
     *
     * @param value
     */
    void invoke(T value) {
        try {
            operation(value)
        }
        catch (ex) {
            logAndThrow(ex)
        }
    }

    /**
     * Method with checked exceptions, takes parameter.
     *
     * @param value
     * @throws Exception
     */
    protected abstract void operation(T value) throws Exception
}